//////////////////////////////////////////////////////////////
//
//  Description	: PDF2HTML/BMP software
//  Author	: verypdf.com Inc
//  Version	: Ver1.2
//  Platform	: Windows95/98/Me/NT/2000/XP
//  Environment	: Visual C++ 6.0
//  Date	: 2002/04/03
//
//  Copyright(C) 2000--2003 verypdf.com Inc
//      
//  Contact infomation:
//        http://www.verypdf.com/
//        support@verypdf.com
//
//  This program is shareware; you can redistribute and/or 
//  modify it under the terms of the GNU General Public License.
//  See the GNU General Public License for more details.
//
//               Portion base on Xpdf code
//         Copyright 1996-2003 Glyph && Cog, LLC
//////////////////////////////////////////////////////////////

#ifdef __GNUC__
#pragma implementation
#endif

#include <aconf.h>
#include "config.h"

#if FREETYPE2 && (HAVE_FREETYPE_FREETYPE_H || HAVE_FREETYPE_H)

#include <math.h>
#include <string.h>
#include "gmem.h"
#include "freetype/internal/ftobjs.h"
#if 1 //~ cff cid->gid map
#include "freetype/internal/cfftypes.h"
#include "freetype/internal/tttypes.h"
#endif
#include "GlobalParams.h"
#include "FTFont.h"
#include <malloc.h>
#include "pdfrlib.h"
#include "GfxState.h"
#include "GfxFont.h"
//------------------------------------------------------------------------

FTFontEngine::FTFontEngine(Display *displayA, Visual *visualA, int depthA,
			   Colormap colormapA, GBool aaA):
  SFontEngine(displayA, visualA, depthA, colormapA) 
  {
	  
	  ok = gFalse;
	  if (FT_Init_FreeType(&lib)) 
	  {
		  return;
	  }
	  aa = aaA;
	  ok = gTrue;
  }

FTFontEngine::~FTFontEngine() 
{
	FT_Done_FreeType(lib);
}

//------------------------------------------------------------------------

FTFontFile::FTFontFile(FTFontEngine *engineA, char *fontFileName,
		       char **fontEnc, GBool pdfFontHasEncoding) 
{
	char *name;
	int unicodeCmap, macRomanCmap, msSymbolCmap;
	int i, j;
	
	ok = gFalse;
	engine = engineA;
	codeMap = NULL;
	if (FT_New_Face(engine->lib, fontFileName, 0, &face)) 
	{
		return;
	}
	
	if (!strcmp(face->driver->root.clazz->module_name, "type1") ||
		!strcmp(face->driver->root.clazz->module_name, "cff")) 
	{
		
		mode = ftFontModeCodeMapDirect;
		codeMap = (Guint *)gmalloc(256 * sizeof(Guint));
		for (i = 0; i < 256; ++i) 
		{
			codeMap[i] = 0;
			if ((name = fontEnc[i])) 
			{
				codeMap[i] = FT_Get_Name_Index(face, name);
			}
		}
		
	}
	else 
	{
		
		// To match up with the Adobe-defined behaviour, we choose a cmap
		// like this:
		// 1. If the PDF font has an encoding:
		//    1a. If the TrueType font has a Microsoft Unicode cmap, use it,
		//        and use the Unicode indexes, not the char codes.
		//    1b. If the TrueType font has a Macintosh Roman cmap, use it,
		//        and reverse map the char names through MacRomanEncoding to
		//        get char codes.
		// 2. If the PDF font does not have an encoding:
		//    2a. If the TrueType font has a Macintosh Roman cmap, use it,
		//        and use char codes directly.
		//    2b. If the TrueType font has a Microsoft Symbol cmap, use it,
		//        and use (0xf000 + char code).
		// 3. If none of these rules apply, use the first cmap and hope for
		//    the best (this shouldn't happen).
		unicodeCmap = macRomanCmap = msSymbolCmap = 0xffff;
		for (i = 0; i < face->num_charmaps; ++i) 
		{
			if (face->charmaps[i]->platform_id == 3 &&
				face->charmaps[i]->encoding_id == 1) 
			{
				unicodeCmap = i;
			}
			else if (face->charmaps[i]->platform_id == 1 &&
				face->charmaps[i]->encoding_id == 0) 
			{
				macRomanCmap = i;
			}
			else if (face->charmaps[i]->platform_id == 3 &&
				face->charmaps[i]->encoding_id == 0) 
			{
				msSymbolCmap = i;
			}
		}
		i = 0;
		mode = ftFontModeCharCode;
		charMapOffset = 0;
		if (pdfFontHasEncoding) 
		{
			if (unicodeCmap != 0xffff) 
			{
				i = unicodeCmap;
				mode = ftFontModeUnicode;
			}
			else if (macRomanCmap != 0xffff) 
			{
				i = macRomanCmap;
				mode = ftFontModeCodeMap;
				codeMap = (Guint *)gmalloc(256 * sizeof(Guint));
				for (j = 0; j < 256; ++j) 
				{
					if (fontEnc[j]) 
					{
						codeMap[j] = globalParams->getMacRomanCharCode(fontEnc[j]);
					}
					else 
					{
						codeMap[j] = 0;
					}
				}
			}
		}
		else 
		{
			if (macRomanCmap != 0xffff) 
			{
				i = macRomanCmap;
				mode = ftFontModeCharCode;
			}
			else if (msSymbolCmap != 0xffff) 
			{
				i = msSymbolCmap;
				mode = ftFontModeCharCodeOffset;
				charMapOffset = 0xf000;
			}
		}
		if (FT_Set_Charmap(face, face->charmaps[i])) 
		{
			return;
		}
	}
	
	ok = gTrue;
}

FTFontFile::FTFontFile(FTFontEngine *engineA, char *fontFileName,
		       Gushort *cidToGIDA, int cidToGIDLenA) 
{
	ok = gFalse;
	engine = engineA;
	codeMap = NULL;
	if (FT_New_Face(engine->lib, fontFileName, 0, &face)) 
	{
		return;
	}
	cidToGID = cidToGIDA;
	cidToGIDLen = cidToGIDLenA;
	mode = ftFontModeCIDToGIDMap;
	ok = gTrue;
}

FTFontFile::FTFontFile(FTFontEngine *engineA, char *fontFileName) 
{
	ok = gFalse;
	engine = engineA;
	codeMap = NULL;
	if (FT_New_Face(engine->lib, fontFileName, 0, &face)) 
	{
		return;
	}
	cidToGID = NULL;
	cidToGIDLen = 0;
	mode = ftFontModeCFFCharset;
	ok = gTrue;
}

FTFontFile::~FTFontFile() 
{
	if (face) {
		FT_Done_Face(face);
	}
	if (codeMap) {
		gfree(codeMap);
	}
}

//------------------------------------------------------------------------

FTFont::FTFont(FTFontFile *fontFileA, double *m) 
{
	FTFontEngine *engine;
	FT_Face face;
	double size, div;
	int x, xMin, xMax;
	int y, yMin, yMax;
	int i;
	
	ok = gFalse;
	fontFile = fontFileA;
	engine = fontFile->engine;
	face = fontFile->face;
	if (FT_New_Size(face, &sizeObj)) 
	{
		return;
	}
	face->size = sizeObj;
	size = sqrt(m[2]*m[2] + m[3]*m[3]);
	if (FT_Set_Pixel_Sizes(face, 0, (int)size)) 
	{
		return;
	}
	
	div = face->bbox.xMax > 20000 ? 65536 : 1;
	
	// transform the four corners of the font bounding box -- the min
	// and max values form the bounding box of the transformed font
	x = (int)((m[0] * face->bbox.xMin + m[2] * face->bbox.yMin) /
		(div * face->units_per_EM));
	xMin = xMax = x;
	y = (int)((m[1] * face->bbox.xMin + m[3] * face->bbox.yMin) /
		(div * face->units_per_EM));
	yMin = yMax = y;
	x = (int)((m[0] * face->bbox.xMin + m[2] * face->bbox.yMax) /
		(div * face->units_per_EM));
	if (x < xMin) {
		xMin = x;
	} else if (x > xMax) {
		xMax = x;
	}
	y = (int)((m[1] * face->bbox.xMin + m[3] * face->bbox.yMax) /
		(div * face->units_per_EM));
	if (y < yMin) {
		yMin = y;
	} else if (y > yMax) {
		yMax = y;
	}
	x = (int)((m[0] * face->bbox.xMax + m[2] * face->bbox.yMin) /
		(div * face->units_per_EM));
	if (x < xMin) {
		xMin = x;
	} else if (x > xMax) {
		xMax = x;
	}
	y = (int)((m[1] * face->bbox.xMax + m[3] * face->bbox.yMin) /
		(div * face->units_per_EM));
	if (y < yMin) {
		yMin = y;
	} else if (y > yMax) {
		yMax = y;
	}
	x = (int)((m[0] * face->bbox.xMax + m[2] * face->bbox.yMax) /
		(div * face->units_per_EM));
	if (x < xMin) {
		xMin = x;
	} else if (x > xMax) {
		xMax = x;
	}
	y = (int)((m[1] * face->bbox.xMax + m[3] * face->bbox.yMax) /
		(div * face->units_per_EM));
	if (y < yMin) {
		yMin = y;
	} else if (y > yMax) {
		yMax = y;
	}
	// This is a kludge: some buggy PDF generators embed fonts with
	// zero bounding boxes.
	if (xMax == xMin) {
		xMin = 0;
		xMax = (int)size;
	}
	if (yMax == yMin) {
		yMin = 0;
		yMax = (int)(1.2 * size);
	}
	// this should be (max - min + 1), but we add some padding to
	// deal with rounding errors
	glyphW = xMax - xMin + 3;
	glyphH = yMax - yMin + 3;
	if (engine->aa) {
		glyphSize = glyphW * glyphH;
	} else {
		glyphSize = ((glyphW + 7) >> 3) * glyphH;
	}
	
	// set up the glyph pixmap cache
	cacheAssoc = 8;
	if (glyphSize <= 256) {
		cacheSets = 8;
	} else if (glyphSize <= 512) {
		cacheSets = 4;
	} else if (glyphSize <= 1024) {
		cacheSets = 2;
	} else {
		cacheSets = 1;
	}
	cache = (Guchar *)gmalloc(cacheSets * cacheAssoc * glyphSize);
	cacheTags = (FTFontCacheTag *)gmalloc(cacheSets * cacheAssoc *
		sizeof(FTFontCacheTag));
	for (i = 0; i < cacheSets * cacheAssoc; ++i) {
		cacheTags[i].mru = i & (cacheAssoc - 1);
	}
	
	// create the XImage
	if (!(image = XCreateImage(engine->display, engine->visual, engine->depth,
		ZPixmap, 0, NULL, glyphW, glyphH, 8, 0))) {
		return;
	}
	image->data = (char *)gmalloc(glyphH * image->bytes_per_line);
	
	// compute the transform matrix
	matrix.xx = (FT_Fixed)((m[0] / size) * 65536);
	matrix.yx = (FT_Fixed)((m[1] / size) * 65536);
	matrix.xy = (FT_Fixed)((m[2] / size) * 65536);
	matrix.yy = (FT_Fixed)((m[3] / size) * 65536);
	
	ok = gTrue;
}

FTFont::~FTFont() 
{
	gfree(cacheTags);
	gfree(cache);
	gfree(image->data);
	image->data = NULL;
	XDestroyImage(image);
}

extern int _XPutImage(Display* a,Drawable hClipRegion,int bitcount,
	      COLORREF  maskColor,
	      XImage*	f	/* image */,
	      int	g		/* src_width */,
	      int	h		/* src_height */,
	      int	i		/* dest_x */,
	      int	j		/* dest_y */,
	      unsigned int k	/* dest_width */,
	      unsigned int l	/* dest_height */,
	      LPSTEXT  pText);
GBool FTFont::drawChar(Drawable d, int w, int h, GC gc,
		       int x, int y, int r, int g, int b,
		       CharCode c, Unicode u) 
{
	FTFontEngine *engine;
	XColor xcolor;
	int bgR, bgG, bgB;
	Gulong colors[5];
	Guchar *p;
	int pix;
	int xOffset, yOffset, x0, y0, x1, y1, gw, gh, w0, h0;
	int xx, yy, xx1;
	
	engine = fontFile->engine;
	
	// generate the glyph pixmap
	if (!(p = getGlyphPixmap(c, u, &xOffset, &yOffset, &gw, &gh))) 
	{
		return gFalse;
	}
	
	// compute: (x0,y0) = position in destination drawable
	//          (x1,y1) = position in glyph image
	//          (w0,h0) = size of image transfer
	x0 = x - xOffset;
	y0 = y - yOffset;
	x1 = 0;
	y1 = 0;
	w0 = gw;
	h0 = gh;
	if (x0 < 0) {
		x1 = -x0;
		w0 += x0;
		x0 = 0;
	}
	if (x0 + w0 > w) {
		w0 = w - x0;
	}
	if (w0 < 0) {
		return gTrue;
	}
	if (y0 < 0) {
		y1 = -y0;
		h0 += y0;
		y0 = 0;
	}
	if (y0 + h0 > h) {
		h0 = h - y0;
	}
	if (h0 < 0) {
		return gTrue;
	}
	
	// read the X image
	XGetSubImage(engine->display, d, x0, y0, w0, h0, (1 << engine->depth) - 1,
		ZPixmap, image, x1, y1);
	
//xueheng add for embed fonts
	r >>= 8;
	g >>= 8;
	b >>= 8;

	if(r == 255 && g == 255 && b == 255)
		b = 250;

	int width = gw;
	int height = gh;
	//int  rowbytes = (width*8+31)/32*4;
	int  rowbytes = (width*24+31)/32*4;
	LPBYTE pData = (LPBYTE)malloc(rowbytes*height);
	memset(pData,255,rowbytes*height);
	LPBYTE pData2;
	if (engine->aa) 
	{
		// compute the colors
		xcolor.pixel = XGetPixel(image, x1, y1);
		XQueryColor(engine->display, engine->colormap, &xcolor);
		//bgR = xcolor.red;
		//bgG = xcolor.green;
		//bgB = xcolor.blue;
		//colors[1] = engine->findColor((r + 3*bgR) / 4,(g + 3*bgG) / 4,(b + 3*bgB) / 4);
		//colors[2] = engine->findColor((r + bgR) / 2,(g + bgG) / 2,(b + bgB) / 2);
		//colors[3] = engine->findColor((3*r + bgR) / 4,(3*g + bgG) / 4,(3*b + bgB) / 4);
		//colors[4] = engine->findColor(r, g, b);
		
		bgR = 255;
		bgG = 255;
		bgB = 255;
		colors[1] = RGB((r + 3*bgR) / 4,(g + 3*bgG) / 4,(b + 3*bgB) / 4);
		colors[2] = RGB((r + bgR) / 2,(g + bgG) / 2,(b + bgB) / 2);
		colors[3] = RGB((3*r + bgR) / 4,(3*g + bgG) / 4,(3*b + bgB) / 4);
		colors[4] = RGB(r, g, b);

		for (yy = 0; yy < gh; ++yy) 
		{
			pData2 = pData+(height-yy-1)*rowbytes;
			for (xx = 0; xx < gw; ++xx) 
			{
				pix = *p++ & 0xff;
				// this is a heuristic which seems to produce decent
				// results -- the linear mapping would be:
				// pix = (pix * 5) / 256;
				pix = ((pix + 10) * 5) / 256;
				if (pix > 4) 
				{
					pix = 4;
				}
				if (pix > 0) 
				{
					//XPutPixel(image, xx, yy, colors[pix]);
					//pData2[xx] = (char)(colors[pix]);
#if 0
					pData2[xx*3+0] = b;
					pData2[xx*3+1] = g;
					pData2[xx*3+2] = r;
#else//ữ
					pData2[xx*3+0] = GetBValue(colors[pix]);
					pData2[xx*3+1] = GetGValue(colors[pix]);
					pData2[xx*3+2] = GetRValue(colors[pix]);
#endif
				}
			}
		}
	}
	else 
	{
		// one color
		colors[1] = RGB(r, g, b);
		for (yy = 0; yy < gh; ++yy) 
		{
			pData2 = pData+(height-yy-1)*rowbytes;
			for (xx = 0; xx < gw; xx += 8) 
			{
				pix = *p++;
				for (xx1 = xx; xx1 < xx + 8 && xx1 < gw; ++xx1) 
				{
					if (pix & 0x80) 
					{
						//XPutPixel(image, xx1, yy, colors[1]);
						//pData2[xx] = (char)(colors[1]);
						pData2[xx1*3+0] = GetBValue(colors[1]);
						pData2[xx1*3+1] = GetGValue(colors[1]);
						pData2[xx1*3+2] = GetRValue(colors[1]);
					}
					pix <<= 1;
				}
			}
		}
		
	}
	
	GfxState *state = (GfxState *)d;
	if(state == NULL)
	{
		free(pData);
		return gFalse;
	}
	char fontname[200];
	int isBold = gFalse,isItalic = gFalse;
	strcpy(fontname,"Courier");
	GfxFont *pFont = state->getFont();
	if(pFont)
	{
		GString *pszFont = pFont->getName();
		if(pszFont)
			strcpy(fontname,pszFont->getCString());

		isItalic = pFont->isItalic();
		isBold   = pFont->isBold();
		if(isItalic == gFalse)
			if(strstr(fontname,"Italic"))
				isItalic = gTrue;
		if(isBold == gFalse)
			if(strstr(fontname,"Bold"))
				isBold = gTrue;
	}
	int nCount = 1;
	STEXT	m_nText;
	m_nText.x	   = x;
	m_nText.y	   = y;
	m_nText.szCode.resize(nCount*2+1);
	memcpy(m_nText.szCode.begin(),&u,nCount*2);
	m_nText.nCount	   = nCount;
	m_nText.szFontName = fontname;
	m_nText.nFontWidth  = w0;
	m_nText.nFontHeight = h0;
	m_nText.m_bIsBold  = isBold;
	m_nText.m_bIsItalic= isItalic;
	m_nText.m_dAngle   = 0;
	m_nText.color	   = RGB(r,g,b);
	m_nText.style	   = 0;
	// draw the X image
//	XPutImage(engine->display, d, gc, image, x1, y1, x0, y0, w0, h0);
	_XPutImage(NULL,NULL,24,
		RGB(255,255,255),
		(XImage *)pData,
		width/* src_width */,
		height/* src_height */,
		x0/* dest_x */,
		y0/* dest_y */,
		w0/* dest_width */,
		h0/* dest_height */,
		&m_nText);
	free(pData);
  
	return gTrue;
}

Guchar *FTFont::getGlyphPixmap(CharCode c, Unicode u,int *x, int *y, int *w, int *h) 
{
	FT_GlyphSlot slot;
	FT_UInt idx;
	int gSize;
	int i, j, k;
	Guchar *ret;
	
	// check the cache
	i = (c & (cacheSets - 1)) * cacheAssoc;
	for (j = 0; j < cacheAssoc; ++j) 
	{
		if ((cacheTags[i+j].mru & 0x8000) && cacheTags[i+j].code == c) 
		{
			*x = cacheTags[i+j].x;
			*y = cacheTags[i+j].y;
			*w = cacheTags[i+j].w;
			*h = cacheTags[i+j].h;
			for (k = 0; k < cacheAssoc; ++k) 
			{
				if (k != j &&
					(cacheTags[i+k].mru & 0x7fff) < (cacheTags[i+j].mru & 0x7fff)) 
				{
					++cacheTags[i+k].mru;
				}
			}
			cacheTags[i+j].mru = 0x8000;
			return cache + (i+j) * glyphSize;
		}
	}
	
	// generate the glyph pixmap or bitmap
	fontFile->face->size = sizeObj;
	FT_Set_Transform(fontFile->face, &matrix, NULL);
	slot = fontFile->face->glyph;
	idx = 0; // make gcc happy
	switch (fontFile->mode) 
	{
	case ftFontModeUnicode:
		idx = FT_Get_Char_Index(fontFile->face, (FT_ULong)u);
		break;
	case ftFontModeCharCode:
		idx = FT_Get_Char_Index(fontFile->face, (FT_ULong)c);
		break;
	case ftFontModeCharCodeOffset:
		idx = FT_Get_Char_Index(fontFile->face,
			(FT_ULong)(c + fontFile->charMapOffset));
		break;
	case ftFontModeCodeMap:
		if (c <= 0xff) 
		{
			idx = FT_Get_Char_Index(fontFile->face, (FT_ULong)fontFile->codeMap[c]);
		}
		else 
		{
			idx = 0;
		}
		break;
	case ftFontModeCodeMapDirect:
		if (c <= 0xff) 
		{
			idx = (FT_UInt)fontFile->codeMap[c];
		}
		else 
		{
			idx = 0;
		}
		break;
	case ftFontModeCIDToGIDMap:
		if (fontFile->cidToGIDLen) 
		{
			if ((int)c < fontFile->cidToGIDLen) 
			{
#ifdef _DEBUG
				int bufsize = _msize( fontFile->cidToGID );
#endif
				idx = (FT_UInt)fontFile->cidToGID[c];
			}
			else 
			{
				idx = (FT_UInt)0;
			}
		}
		else 
		{
			idx = (FT_UInt)c;
		}
		break;
	case ftFontModeCFFCharset:
#if 1 //~ cff cid->gid map
		CFF_Font *cff = (CFF_Font *)((TT_Face)fontFile->face)->extra.data;
		idx = 0;
		for (j = 0; j < (int)cff->num_glyphs; ++j) 
		{
			if (cff->charset.sids[j] == c) 
			{
				idx = j;
				break;
			}
		}
#endif
		break;
	}
#if 1
	//~ the hinting in FT 2.0.6 appears to have some problems, so turn
	//~ it off if anti-aliasing is enabled; if anti-aliasing is disabled,
	//~ this seems to be a tossup - some fonts look better with hinting,
	//~ some without, so leave hinting on
	if (FT_Load_Glyph(fontFile->face, idx,
		fontFile->engine->aa ? FT_LOAD_NO_HINTING : FT_LOAD_DEFAULT) ||
#else
		if (FT_Load_Glyph(fontFile->face, idx, FT_LOAD_DEFAULT) ||
#endif
			FT_Render_Glyph(slot,
			fontFile->engine->aa ? ft_render_mode_normal : ft_render_mode_mono)) 
		{
			return gFalse;
		}
		*x = -slot->bitmap_left;
		*y = slot->bitmap_top;
		*w = slot->bitmap.width;
		*h = slot->bitmap.rows;
		if (*w > glyphW || *h > glyphH) 
		{
#if 1 //~ debug
			fprintf(stderr, "Weird FreeType glyph size: %d > %d or %d > %d\n",
				*w, glyphW, *h, glyphH);
#endif
			return NULL;
		}
		
		// store glyph pixmap in cache
		ret = NULL;
		for (j = 0; j < cacheAssoc; ++j) 
		{
			if ((cacheTags[i+j].mru & 0x7fff) == cacheAssoc - 1) 
			{
				cacheTags[i+j].mru = 0x8000;
				cacheTags[i+j].code = c;
				cacheTags[i+j].x = *x;
				cacheTags[i+j].y = *y;
				cacheTags[i+j].w = *w;
				cacheTags[i+j].h = *h;
				if (fontFile->engine->aa) 
				{
					gSize = *w * *h;
				}
				else 
				{
					gSize = ((*w + 7) >> 3) * *h;
				}
				ret = cache + (i+j) * glyphSize;
				memcpy(ret, slot->bitmap.buffer, gSize);
			}
			else 
			{
				++cacheTags[i+j].mru;
			}
		}
		return ret;
}

#endif // FREETYPE2 && (HAVE_FREETYPE_FREETYPE_H || HAVE_FREETYPE_H)
